// audio.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"


// was:
// daytime_client.cpp
// ~~~~~~~~~~~~~~~~~~
// Copyright (c) 2003-2011 Christopher M. Kohlhoff (chris at kohlhoff dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#include "logger.hpp"
#include "stream_socket_service.hpp"

typedef ba::basic_stream_socket<boost::asio::ip::tcp,
    services::stream_socket_service<boost::asio::ip::tcp> > debug_stream_socket;


std::fstream* pOS= NULL;


enum HTTPRQ
{
	HTTPRQ_OK,
	HTTPRQ_UNQ,
};

namespace boost{ namespace asio {

enum request_type
{
	HTTP_GET,
	RTSP_SETUP,
};

}}//namespace

void get_request_buf( std::string& client, std::string& path, ba::streambuf& sbuf, ba::request_type type )
{
	std::ostream request_stream( &sbuf );
	switch( type )
	{
	case ba::HTTP_GET:

		request_stream << "GET " << path << " HTTP/1.0\r\n";
		request_stream << "Host: " << client << "\r\n";
		request_stream << "Accept: */*\r\n";
		//request_stream << "Icy-MetaData:1\r\n";
//		request_stream << "Connection: close\r\n";
		request_stream << "\r\n";
		return;

	case ba::RTSP_SETUP:

		request_stream << "SETUP http://" << client << path << " RTSP/1.0\r\n";
		request_stream << "Cseq: 1\r\n";
		request_stream << "Transport: RTP/AVP/UDP; unicast; destination=168.192.0.0; client_port=32884\r\n\r\n";
		return;

	//
//		request_stream << "DESCRIBE " << client << path << " RTSP/1.0\r\nCSeq: 2\r\n\r\n";
	}//switch
}


//can use a complete url as: http://example.org/folder/thepage.html
//http://riosal.ic.llnwd.net/stream/riosal_kjzz.m3u
//rtsp://riosal.ic.llnwd.net/stream/riosal_kjzz.m3u
//if http the 'http://' can be left off, it will be assumed;

int host_request( std::tstring& url, ba::ip::tcp::socket& socket, boost::asio::streambuf& response, ba::request_type type= ba::HTTP_GET );
int host_request( std::tstring& url, ba::ip::tcp::socket& socket, boost::asio::streambuf& response, ba::request_type type )
{
	std::string i_url( url );
	if( url.substr( 0, 7 ) == "http://" || url.substr( 0, 7 ) == "rtsp://" )
		i_url.erase( i_url.begin( ), i_url.begin( ) + 7 );

	std::string::size_type to= i_url.find( "/" );
	std::tstring client;
	std::tstring path;
	if( to != std::string::npos )
	{
		client.assign( i_url.begin( ), i_url.begin( ) + to );
		path.assign( i_url.begin( ) + to, i_url.end( ) );
	}
	else
	{
		client= i_url;
		path= "/";
	}
	//do we have a port # ?
	std::string port;
	to= client.find( ":" );
	if( to != std::string::npos )
	{
		port.assign( client.begin( ) + to + 1, client.end( ) );
		client.erase( client.begin( ) + to, client.end( )  );
	}
	else
		port= "http";

	try
	{
		ba::ip::tcp::resolver resolver( socket.get_io_service( ) );
		ba::ip::tcp::resolver::query query( client, port );
		ba::ip::tcp::resolver::iterator iterator = resolver.resolve( query );

		ba::ip::tcp::resolver::iterator end;
		ba::ip::tcp::resolver::iterator endpoint_iterator = resolver.resolve( query );

		// Try each endpoint until we successfully establish a connection.
		boost::system::error_code error = boost::asio::error::host_not_found;
		for( ; error && endpoint_iterator != end; )
		{
			socket.close( );
			socket.connect( *endpoint_iterator++, error );
		}
		if( error )
		  throw boost::system::system_error( error );

		// Form the request
		ba::streambuf request;
		get_request_buf( client, path, request, type );

		// Send the request.
		ba::write( socket, request );

		// Read the response.
		ba::read_until( socket, response, "\r\n\r\n" );

		// Check that response is OK.
		std::istream response_stream(&response);
		std::tstring http_version;
		response_stream >> http_version;
		std::cout << http_version << std::endl;
		unsigned int status_code;
		response_stream >> status_code;
		std::cout << status_code << std::endl;
		std::tstring status_message;
		std::getline(response_stream, status_message);
		std::cout << status_message << std::endl;

		if( http_version.substr(0, 6) != "HTTP/1" )
			return HTTPRQ_UNQ;

		if( status_code != 200 )
			return HTTPRQ_UNQ;

	}
	catch( std::exception& e )
	{
		std::cerr << "~~http_request failed: " << e.what( ) << std::endl;
		std::tstring msg = "Test Failed: ";
		msg+= e.what( );

		return HTTPRQ_UNQ;
	}
	std::cout << "~~~HTTP OK" << std::endl;
	return HTTPRQ_OK;
}

int main(int argc, char* argv[])
{

	ba::io_service io_service;
	ba::ip::tcp::socket sock( io_service );
    services::logger logger(io_service, "");
    logger.use_file("log.txt");
	{
		std::tstring msg = "Begin Test";
		logger.log( msg );
	}

	try
	{

		ba::streambuf response;
		std::istream response_stream( &response );

		std::string url;

		//thi works as expected
		//url="http://www.amazon.com";

		//TODO dig url from this response to get next url
		url= "http://riosal.ic.llnwd.net/stream/riosal_kjzz.m3u";
		//url= "http://sc4.dal.llnw.net:80/stream/riosal_kjzz";

		//url= "http://streaming.gmgradio.com/rockradioscotland.m3u";
		//url= "http://shoutcast.gmgradio.com:10001";

		//url= "http://205.188.215.228:8002/";

		//url= "http://50.117.121.12:80";
		//url= "http://7609.live.streamtheworld.com:80/977_CLASSROCKAAC_SC";

		std::cout << "path: " << url << "\n\n";
		if( host_request( url, sock, response, ba::HTTP_GET ) != HTTPRQ_OK )
		{
			std::cout << "~~~Error\n";
		}

		std::string str;
		for( ; ! response_stream.eof( ); )
		{
			std::getline( response_stream, str );
			std::cout << str << std::endl;
			if( str == "\r" )
			{
				std::cout << "\nHeader Finished\n";
				//break;
			}
		}
		//for the likes of http://riosal.ic.llnwd.net/stream/riosal_kjzz.m3u
		//there should be just one more line after the header, the stream URL

		std::fstream os;
		os.open( "C:\\Users\\Dan\\test.mp3", std::fstream::trunc | std::fstream::out );
		pOS= &os;

		std::vector< char > buf( 50000 );
		char* pBuf= &buf.front( );

		boost::system::error_code ec;
		for( int i=0 ; i < 100 ; ++i )
		{
			__int64 r= sock.read_some( ba::buffer( pBuf, 50000 ), ec );
			os.write( pBuf, r );
			std::cout << " ~" << r;
			if( ec )
				break;
		}
		os.close( );
		int a= 0;//for break point
	}
	catch( std::exception& e )
	{
		std::cerr << e.what( ) << std::endl;
		std::tstring msg = "Test Failed: ";
		msg+= e.what( );
		logger.log( msg );
#ifdef _DEBUG
		getchar( );
#endif
	}
#ifdef _DEBUG
	getchar( );
#endif
	return 0;
}


//void read_handler(const boost::system::error_code& e,
//    std::size_t bytes_transferred, debug_stream_socket* s)
//{
//  if (!e)
//  {
//	  std::cout.write(read_buffer.c_array( ), bytes_transferred);
//
//    s->async_read_some(ba::buffer(read_buffer),
//        boost::bind(read_handler, ba::placeholders::error,
//          ba::placeholders::bytes_transferred, s));
//  }
//}
//
//void connect_handler(const boost::system::error_code& e, debug_stream_socket* s)
//{
//  if( ! e )
//  {
//    s->async_read_some( ba::buffer( read_buffer ),
//        boost::bind( read_handler, ba::placeholders::error,
//          ba::placeholders::bytes_transferred, s
//		 )
//	);
//
//  }
//  else
//  {
//    std::cerr << e.message() << std::endl;
//  }
//}
//
//void resolve_handler(const boost::system::error_code &ec, boost::asio::ip::tcp::resolver::iterator it) 
//{ 
//  if (!ec) 
//  { 
//    sock.async_connect( *it, connect_handler); 
//  } 
//}


